GET_API_TOKEN_VALIDATION_V1 = "v1/accessControl/apitoken/validate"           # GET – Obtain organization and project information related to the provided apitoken.
GET_AUTHORIZED_ORGANIZATIONS_V1 = "v1/accessControl/user/organizations"      # GET - Obtain the list of organizations that a user is permitted to access.
GET_AUTHORIZED_PROJECTS_V1 = "v1//accessControl/user/organization/projects"  # GET - Obtain the list of projects that a user is permitted to access in a particular organization.
GET_PROJECT_VISIBILITY_V1 = "v1/accessControl/user/access/project"          # GET - Determines if a GAM user has visibility of a project
GET_PROJECT_API_TOKEN_V1 = "v1/accessControl/user/token/project"            # GET - Returns Project's API Token

